/*
 *  Copyright (c) 2012 Matisse Software Inc. All Rights Reserved.
 *
 *  This file (in both binary and source code form) is subject to the
 *  Supplemental License Terms governing use, modification and redistribution
 *  of Sample Code accompanying the Matisse(R) software.
 *
 */

#import <Foundation/Foundation.h>

// Matisse Objective-C header
#import "matisseObjC.h"

#import "Person.h"
#import "Employee.h"
#import "Manager.h"
#import "PostalAddress.h"


// small number for demo purpose
static int DEFAULT_ALLOCATOR_CNT = 50;
static int SAMPLE_OBJECT_CNT = 100;
static int OBJECT_PER_TRAN_CNT = 20;

// A simple example which shows how to load a large number of objects
int main(int argc, char **argv)
{
  MtDatabase* db;
  NSString *hostname, *dbname;
  int i;

  if (argc < 3) {
    printf("Usage: %s <host> <database>\n", argv[0]);
    return -1;
  }
#if !__has_feature(objc_arc)
  NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
#else
  @autoreleasepool {
#endif

  hostname = [NSString stringWithCString:argv[1] encoding:NSUTF8StringEncoding]; 
  dbname = [NSString stringWithCString:argv[2] encoding:NSUTF8StringEncoding];

  @try {

    // The third argument to MtDatabase init is given so that the connection 
    // object can find the mapping between the Objective-C class Person and 
    // the schema class Person defined in the "examples.objc_examples.chap_10"
    // namespace.
    //
    MtDynamicObjectFactory *factory = [[MtDynamicObjectFactory alloc] init:@"" :@"examples.objc_examples.chap_10"];
    db = [(MtDatabase *)[MtDatabase alloc] init:hostname :dbname andFactory:factory];

    [db open];

    [db startTransaction];

    // Optimize the objects loading
    // Preallocate OIDs so objects can be created in the client workspace 
    // without requesting any further information from the server
    [db preallocate:DEFAULT_ALLOCATOR_CNT];
    Employee *e = nil;
    Manager *m = nil;
    NSString* fname;
    NSString* lname;
    NSDate *ts;
    NSDecimalNumber* num;
    PostalAddress *a;

    for (i = 0; i < SAMPLE_OBJECT_CNT; i++) {
      if (!m || (i % 10 == 0)) {
	m = [[Manager alloc] init: db];
	// set attributes
	fname = [[NSString alloc] initWithFormat:((i % 2 == 0) ?  @"Franck%d" : @"Phil%d"), (i+1)];
	lname = ((i % 2 == 0) ? @"Willianson" : @"Jefferson");
	
	[m setFirstName:fname];
	[m setLastName:lname];
	[m setAge:(21 + (i % 44))];
	ts = [NSDate dateWithString:@"2012-06-06 00:00:00 +0000" ];
	[m setHireDate:ts];
	num = [NSDecimalNumber decimalNumberWithString:@"12345.00"];
	[m setSalary:num];

	a = [[PostalAddress alloc] init:db];
	[a setCity:((i % 2 == 0) ? @"Portland" : @"Stuttgart")];
	[a setPostalCode:((i % 2 == 0) ? @"04179" : @"24100")];
		
	[m setAddress:a];
	printf("%s object created\n", [[m description] UTF8String]);

      }
      // create a new Employee
      e = [[Employee alloc] init:db];
      // set attributes
      fname = [[NSString alloc] initWithFormat:((i % 2 == 0) ?  @"Marty%d" : @"Gerry%d"), (i+1)];
      lname = ((i % 2 == 0) ? @"Paulson" : @"Jackson");
      [e setFirstName:fname];
      [e setLastName:lname];
      [e setAge:(21 + (i % 44))];
      ts = [NSDate dateWithString:@"2012-06-06 00:00:00 +0000" ];
      [e setHireDate : ts];
      num = [NSDecimalNumber decimalNumberWithString:@"12345.00"];
      [e setSalary: num];

      a = [[PostalAddress alloc] init:db];
      [a setCity:((i % 2 == 0) ? @"Portland" : @"Stuttgart")];
      [a setPostalCode:((i % 2 == 0) ? @"04179" : @"24100")];
		
      [e setAddress:a];

      [e setReportsTo:m];
      printf("%s object created\n", [[e description] UTF8String]);

      if (i % OBJECT_PER_TRAN_CNT == 0) {
	[db commit];
	[db startTransaction];
      }
      // check the remaining number of preallocated objects.
      if ([db numPreallocated] < 2) {
	[db preallocate:DEFAULT_ALLOCATOR_CNT];
      }
    }

    if ([db isTransactionInProgress]) {
      [db commit];
    }

    [db startVersionAccess];

    // list all Persons
    printf("\nEnumerating objects...\n");
    printf("%d Persons in the database %s\n", [Person instanceNumber:db],
	   [[db description] UTF8String]);
    printf("%d Employees in the database %s\n", [Employee ownInstanceNumber:db],
	   [[db description] UTF8String]);
    printf("%d Managers in the database %s\n", [Manager instanceNumber:db],
	   [[db description] UTF8String]);

    // open an iterator on all the instances of Manager
    Manager* x;
    Employee* y;
    MtObjectEnumerator* piter = [Manager instanceEnumerator:db];
    while ((x = (Manager*)[piter nextObject]) != nil) {
      // show attributes and name of class
      printf("%s %s is a %s\n", 
	     [[x firstName] UTF8String],
	     [[x lastName] UTF8String],
	     [[[x getMtClass] mtFullName] UTF8String]);

      MtObjectArray* team = [x team];
      printf("Team of %d members:\n", [team count]);
      // load in the client cache all the team members at once
      [team load];
      for (i = 0; i < [team count]; i++) {
	y = (Employee*)[team objectAtIndex:i];
	printf("    %s %s is a %s\n", 
	       [[y firstName] UTF8String],
	       [[y lastName] UTF8String],
	       [[[y getMtClass] mtFullName] UTF8String]);
      }
      // unload from the client cache all the team members at once
      [team unload];

    }
    [piter close];
    [db endVersionAccess];

    [db startTransaction:MT_MAX_TRAN_PRIORITY];
    printf("\nRemoving created objects...\n");

    [[Person getClass:db] removeAllInstances];

    printf("%d Person(s) in the database %s\n", [Person instanceNumber:db],
	   [[db description] UTF8String]);

    [db commit];

    [db close];

  }
  @catch (MtException *e) {
    printf("ERROR: %s: (code=%d) %s\n", 
	   [[e name] UTF8String], 
	   [e errorCode], 
	   [[e message] UTF8String] );
    NSLog(@"Stack trace: %@\n", [e callStackSymbols]);
    if ([db isTransactionInProgress]) {
      [db rollback];
    }
    if ([db isConnectionOpen]) {
      [db close];
    }
  }
  @finally {
#if !__has_feature(objc_arc)
    [db release];
#endif
  }
#if !__has_feature(objc_arc)
  // free pool
  [pool release];
#else
  } // @autoreleasepool
#endif

  printf("Done\n");

  return 0;
}
