/*
 *  Copyright (c) 2012 Matisse Software Inc. All Rights Reserved.
 *
 *  This file (in both binary and source code form) is subject to the
 *  Supplemental License Terms governing use, modification and redistribution
 *  of Sample Code accompanying the Matisse(R) software.
 *
 */

#import <Foundation/Foundation.h>

// Matisse Objective-C header
#import "matisseObjC.h"

#import "Person.h"
#import "Employee.h"
#import "Manager.h"

// A simple example which shows how to use generated code for user classes.
int main(int argc, char **argv)
{
  MtDatabase* db;
  NSString *hostname, *dbname;
  
  if (argc < 3) {
    printf("Usage: %s <host> <database>\n", argv[0]);
    return -1;
  }
#if !__has_feature(objc_arc)
  NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
#else
  @autoreleasepool {
#endif

  hostname = [NSString stringWithCString:argv[1] encoding:NSUTF8StringEncoding]; 
  dbname = [NSString stringWithCString:argv[2] encoding:NSUTF8StringEncoding];

  @try {

    // The third argument to MtDatabase init is given so that the connection 
    // object can find the mapping between the Objective-C class Person and 
    // the schema class Person defined in the "examples.objc_examples.chaps_6_7_8"
    // namespace.
    //
    MtDynamicObjectFactory *factory = [[MtDynamicObjectFactory alloc] init:@"" :@"examples.objc_examples.chaps_6_7_8"];
    db = [(MtDatabase *)[MtDatabase alloc] init:hostname :dbname andFactory:factory];

    [db open];

    [db startTransaction];

    // create a Manager
    Manager* m1 = [[Manager alloc] init:db];
    [m1 setFirstName:@"J.C."];
    [m1 setLastName:@"Dithers"];
    NSDate *ts = [NSDate dateWithString:@"2012-01-06 00:00:00 +0000"];
    [m1 setHireDate:ts];
     NSDecimalNumber* num = [NSDecimalNumber decimalNumberWithString:@"1234.50"];
    [m1 setSalary:num];
    // set the successor object for reportsTo (in this case it
    // refers to itself, i.e., this manager is the big boss at
    // the top of the reporting hierarchy)
    [m1 setReportsTo:m1];

    // create another Manager
    Manager* m2 = [[Manager alloc] init:db];
    [m2 setFirstName:@"Dagwood"];
    [m2 setLastName:@"Bumstead"];
    ts = [NSDate dateWithString:@"2012-02-06 00:00:00 +0000"];
    [m2 setHireDate:ts];
    num = [NSDecimalNumber decimalNumberWithString:@"1234.50"];
    [m2 setSalary:num];
    [m2 setReportsTo:m1]; // the manager

    // create a new Employee
    Employee* e = [[Employee alloc] init: db];
    [e setFirstName: @"Elmo"];
    [e setLastName: @"Tuttle"];
    ts = [NSDate dateWithString : @"2012-03-06 00:00:00 +0000" ];
    [e setHireDate : ts];
    num = [NSDecimalNumber decimalNumberWithString : @"1000.50"];
    [e setSalary: num];
    [e setReportsTo:m2];

    // specify an assistant for each of the two managers
    [m1 setAssistant:e];
    [m2 setAssistant:e];

    // check the assistantOf from the employee
    printf("\nInitial settings:\n");
    printf("successor list:\n");
    MtSize i;
    MtObjectArray* mgrs = [e assistantOf];
    for (i = 0; i < [mgrs count]; i++) {
      Manager* m = (Manager*)[mgrs objectAtIndex:i];
      // show attributes and name of class
      printf("%s is %s's assistant\n", 
	     [[e firstName] UTF8String],
	     [[m firstName] UTF8String]);
    }
    printf("successor list to array:\n");
    for (Manager* mm in [mgrs toArray]) {
      // show attributes and name of class
      printf("%s is %s's assistant\n", 
	     [[e firstName] UTF8String],
	     [[mm firstName] UTF8String]);
    }
    printf("successor enumerator:\n");
    Manager* x;
    MtObjectEnumerator* it = [e assistantOfEnumerator];
    while ((x = (Manager*)[it nextObject]) != nil) {
      // show attributes and name of class
      printf("%s is %s's assistant\n", 
	     [[e firstName] UTF8String],
	     [[x firstName] UTF8String]);
    }
    [it close];

    // create a few extra persons
    Person* c1 = [[Person alloc] init: db];
    [c1 setFirstName: @"Alexander"];
    [c1 setLastName: @"Bumstead"];

    Person* c2 = [[Person alloc] init: db];
    [c2 setFirstName: @"Cookie"];
    [c2 setLastName: @"Bumstead"];

    NSArray* childarray = [NSArray arrayWithObjects:c1, c2, nil];
    MtObjectArray* children = [[MtObjectArray alloc] initWithArray:db :childarray];
    [m2 setChildren:children];

    // When accessing a relationship with single cardinality [1] or
    // [0,1], the access method returns a pointer to the object so
    // that it can be checked for NULL.
    // in this case [Person father] returns Person*
    printf("Set successors:\n");
    printf("%s is %s's child\n", 
	   [[c1 firstName] UTF8String],
	   [[[c1 father] firstName] UTF8String]);
    printf("%s is %s's child\n", 
	   [[c2 firstName] UTF8String],
	   [[[c2 father] firstName] UTF8String]);

    // convenient use of C overloaded operators
    printf("\nCompare objects:\n");

    printf("Do %s (hash=%ld) and %s (hash=%ld) have the same father?\n",
	   [[c1 firstName] UTF8String],
	   [[c1 firstName] hash],
	   [[c2 firstName] UTF8String],
	   [[c2 firstName] hash]);
    printf("(use ==)? ");
   if ([c1 father] == [c2 father]) {
      printf("YES!\n");
    } else {
      printf("NO!!!\n");
    }
   printf("(use isEqual:)? ");
    if ([[c1 father] isEqual:[c2 father]]) {
      printf("YES!\n");
    } else {
      printf("NO!!!\n");
    }

    // the following code shows how to add and remove successors 
    Person* c3 = [[Person alloc] init: db];
    [c3 setFirstName: @"Baby"];
    [c3 setLastName: @"Bumstead"];

    // append to existing list
    [m2 appendChildren:c3];
    printf("\nAdd another successor:\n");
    printf("Now %s has %d children\n", 
	   [[m2 firstName] UTF8String], [m2 childrenSize]);
  
    // use of MtObjectArray for specifying a list to remove
    NSArray* childarray2 = [NSArray arrayWithObjects:c3, c2, nil];
    MtObjectArray* children2 = [[MtObjectArray alloc] initWithArray:db :childarray2];
    // remove 
    [m2 removeNumChildren:children2];
   printf("\nRemove two successors:\n");

     // another signature supports a single removal 
    //  i.e.    [m2 removeChildren:c2];

    printf("Now %s has %d children\n", 
	   [[m2 firstName] UTF8String], [m2 childrenSize]);

    // an alternative to get the relationship size 
    // but the object oid list is loaded before you can get the count

    printf("Now %s has %d children\n", 
	   [[m2 firstName] UTF8String], [[m2 children] count]);

    // clear all the successors
    [m2 clearChildren];
    printf("\nClear successors:\n");
    printf("Now %s has %d children\n", 
	   [[m2 firstName] UTF8String], [m2 childrenSize]);

    // reset to original array
    [m2 appendChildren:c1];
    [m2 appendChildren:c2];
    [m2 appendChildren:c3];

    printf("\nAdd successors and iterate through children...\n");
    printf("Now %s has %d children\n", 
	   [[m2 firstName] UTF8String], [m2 childrenSize]);
    Person *c4;
    MtObjectEnumerator* cit = [m2 childrenEnumerator];
    while ((c4 = (Person*)[cit nextObject]) != nil) {
      // show attributes and name of class
      printf("%s is %s's father\n", 
	     [[m2 firstName] UTF8String],
	     [[c4 firstName] UTF8String]);
    }
    [cit close];


    [db commit];


    [db startTransaction:MT_MAX_TRAN_PRIORITY];

    //remove created objects
    [m1 remove];
    [m2 remove];
    [e remove];
    [c1 remove];
    [c2 remove];
    [c3 remove];

    [db commit];


    [db close];

  }
  @catch (MtException *e) {
    printf("ERROR: %s: (code=%d) %s\n", 
	   [[e name] UTF8String], 
	   [e errorCode], 
	   [[e message] UTF8String] );
    NSLog(@"Stack trace: %@\n", [e callStackSymbols]);
  }
  @finally {
#if !__has_feature(objc_arc)
    [db release];
#endif
  }
#if !__has_feature(objc_arc)
  // free pool
  [pool release];
#else
  } // @autoreleasepool
#endif

  printf("Done\n");

  return 0;
}
