/*
 *  Copyright (c) 2012 Matisse Software Inc. All Rights Reserved.
 *
 *  This file (in both binary and source code form) is subject to the
 *  Supplemental License Terms governing use, modification and redistribution
 *  of Sample Code accompanying the Matisse(R) software.
 *
 */

#import <Foundation/Foundation.h>

// Matisse Objective-C header
#import "matisseObjC.h"

/**
 * This examples illustrates Matisse Event Notification mechansim.
 * The sample application is divided in 2 sections. The first section 
 * is event selection and notification. The second section is event registration 
 * and event handling.
 */


const int TEMPERATURE_CHANGES_EVT = MT_EVENT1;
const int RAINFALL_CHANGES_EVT = MT_EVENT2;
const int HIMIDITY_CHANGES_EVT = MT_EVENT3;
const int WINDSPEED_CHANGES_EVT = MT_EVENT4;

const unsigned int MAX_MEASURES = 12;
static double tempMeasures[] = { 48.7, 52.2, 53.3, 55.6, 58.1, 61.5, 62.7, 63.7, 64.5, 61.0, 54.8, 49.4 };
static double rainfallMeasures[] = { 4.72, 4.15, 3.40, 1.25, 0.54, 0.13, 0.04, 0.09, 0.28, 1.19, 3.31, 3.18 };

static bool ChangeInTemperature(int idx)
{
  bool res = true;

  if (idx > 0) {
    double absVal = (tempMeasures[idx] > tempMeasures[idx - 1] ? 
		     tempMeasures[idx] - tempMeasures[idx - 1] : 
		     tempMeasures[idx - 1] - tempMeasures[idx]);
    res = (absVal > 2.5);
  }
  return res;
}

static bool ChangeInRainfall(int idx)
{
  bool res = true;

  if (idx > 0) {
    double absVal = ((rainfallMeasures[idx] > rainfallMeasures[idx - 1]) ?
		     (rainfallMeasures[idx] - rainfallMeasures[idx - 1]) :
		     (rainfallMeasures[idx - 1] - rainfallMeasures[idx]));
      res = (absVal > 0.25);
    }
  return res;
}

/** Execute an event selection and notification */
static void NotifyOfChanges(NSString* hostname, NSString* dbname)
{
  MtDatabase* db;
  MtEvent* notifier;
  unsigned int i;

  printf("\n\n=========== Notify Of Changes ==========\n");

  @try {

    // wait a little ...
    printf("waiting 3 seconds one or more subscribers can be started ...\n");
    sleep(3);

    db = [(MtDatabase *)[MtDatabase alloc] init:hostname :dbname];

    [db open];

    notifier = [[MtEvent alloc] init:db];
   
    MtEvents eventSet;

    for (i = 0; i < MAX_MEASURES; i++) {
	eventSet = 0;
	if (ChangeInTemperature(i)) eventSet |= TEMPERATURE_CHANGES_EVT;
	if (ChangeInRainfall(i)) eventSet |= RAINFALL_CHANGES_EVT;
	if (i % 2 == 0) eventSet |= HIMIDITY_CHANGES_EVT;
	if (i % 2 == 1) eventSet |= WINDSPEED_CHANGES_EVT;

	[notifier notify:eventSet];

	printf("Events (#%d) posted:\n", (i+1));
	if ((eventSet & TEMPERATURE_CHANGES_EVT) > 0) printf("Change in temperature\n");
	if ((eventSet & RAINFALL_CHANGES_EVT) > 0) printf("Change in rain fall\n");
	if ((eventSet & HIMIDITY_CHANGES_EVT) > 0) printf("Change in humidity\n");
	if ((eventSet & WINDSPEED_CHANGES_EVT) > 0) printf("Change in wind speed\n");

	// wait a little ...
	printf("waiting 1 sec ...\n");
	sleep(1);
      }

    [db close];

  }
  @catch (MtException *e) {
    printf("ERROR: %s: (code=%d) %s\n", 
	   [[e name] UTF8String], 
	   [e errorCode], 
	   [[e message] UTF8String] );
    NSLog(@"Stack trace: %@\n", [e callStackSymbols]);
  }
  @finally {
    [notifier release];
    [db release];
  }

}


/** Execute an event selection and notification */
static void SubscribeToChanges(NSString* hostname, NSString* dbname)
{
  MtDatabase* db;
  MtEvent* subscriber;

  printf("\n\n=========== Subscribe To Changes ==========\n");
  @try {

    db = [(MtDatabase *)[MtDatabase alloc] init:hostname :dbname];

    [db open];

    subscriber = [[MtEvent alloc] init:db];
   
     // Subscribe to all 4 events
    MtEvents eventSet = TEMPERATURE_CHANGES_EVT |
      RAINFALL_CHANGES_EVT |
      HIMIDITY_CHANGES_EVT |
      WINDSPEED_CHANGES_EVT;

    [subscriber subscribe:eventSet];

    printf("Subscribed to 4 Events:\n");
    printf("- TEMPERATURE_CHANGES\n");
    printf("- RAINFALL_CHANGES\n");
    printf("- HIMIDITY_CHANGES\n");
    printf("- WINDSPEED_CHANGES\n");

    MtEvents triggeredEvents;
    int i = 1;
    while (i < 20) {
      printf("Wait for Events (#%d) ...\n", i);
      // Wait 1000 ms for events to be triggered 
      // return false if not event is triggered until the timeout is reached
      if ([subscriber wait:1090  :&triggeredEvents]) {
	printf("Events (#%d) triggered:\n"\
	       "%sChange in temperature\n"\
	       "%sChange in rain fall\n"\
	       "%sChange in humidity\n"\
	       "%sChange in wind speed\n", i,
	       (((triggeredEvents & TEMPERATURE_CHANGES_EVT) > 0) ? "" : "No "),
	       (((triggeredEvents & RAINFALL_CHANGES_EVT) > 0) ? "" : "No "),
	       (((triggeredEvents & HIMIDITY_CHANGES_EVT) > 0) ? "" : "No "),
	       (((triggeredEvents & WINDSPEED_CHANGES_EVT) > 0) ? "" : "No "));
      } else {
	printf("No Event received after ~1 sec\n");
      }
      i++;
    }
    
    printf("Unsubscribe to 4 Events\n");
    // Unsubscribe to all 4 events
    [subscriber unsubscribe];

    [db close];

  }
  @catch (MtException *e) {
    printf("ERROR: %s: (code=%d) %s\n", 
	   [[e name] UTF8String], 
	   [e errorCode], 
	   [[e message] UTF8String] );
    NSLog(@"Stack trace: %@\n", [e callStackSymbols]);
  }
  @finally {
    [subscriber release];
    [db release];
  }
}

int main(int argc, char **argv)
{
  NSString *hostname, *dbname;
  
  if (argc < 4) {
    printf("Usage: %s <host> <database> N|S\n", argv[0]);
    return -1;
  }
#if !__has_feature(objc_arc)
  NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
#else
  @autoreleasepool {
#endif

  hostname = [NSString stringWithCString:argv[1] encoding:NSUTF8StringEncoding]; 
  dbname = [NSString stringWithCString:argv[2] encoding:NSUTF8StringEncoding];

  if (strcmp("N", argv[3]) == 0) {
    // event selection and notification
    NotifyOfChanges(hostname, dbname);
  } else {
    // event registration and event handling
    SubscribeToChanges(hostname, dbname);
  }

#if !__has_feature(objc_arc)
  // free pool
  [pool release];
#else
  } // @autoreleasepool
#endif

  printf("Done\n");

  return 0;
}
