/*
 *  Copyright (c) 2012 Matisse Software Inc. All Rights Reserved.
 *
 *  This file (in both binary and source code form) is subject to the
 *  Supplemental License Terms governing use, modification and redistribution
 *  of Sample Code accompanying the Matisse(R) software.
 *
 */

#import <Foundation/Foundation.h>

// Matisse Objective-C header
#import "matisseObjC.h"

#import "Person.h"
#import "Employee.h"
#import "Manager.h"

// this example shows how to access with the same application 2 logical databases
// into the database physical database server.


void createObjects(NSString *hostname, NSString *dbname, NSString *dbnamespace, int minId)
{
  MtDatabase* db;
  NSDate *ts;
  NSDecimalNumber *num;
  Person* x;

  printf("=========== Create Objects ===========\n");

  @try {

    // The third argument to MtDatabase init is given so that the connection 
    // object can find the mapping between the Objective-C class Person and 
    // the schema class Person defined in the namespace defined by the 
    // 'dbnamespace' parameter.
    //
    MtDynamicObjectFactory *factory = [[MtDynamicObjectFactory alloc] init:@"" :dbnamespace];
    db = [(MtDatabase *)[MtDatabase alloc] init:hostname :dbname andFactory:factory];

    [db open];

    [db startTransaction];

    // create a new Person
    Person* p1 = [[Person alloc] init:db];
    [p1 setId:minId+1];
    [p1 setFirstName:@"John"];
    [p1 setLastName:@"Smith"];
    [p1 setAge:43];

    // create a new Employee
    Employee* e = [[Employee alloc] init: db];
    [e setId:minId+2];
    [e setFirstName: @"Jane"];
    [e setLastName: @"Jones"];
    ts = [NSDate dateWithString : @"2012-03-06 00:00:00 +0000" ];
    [e setHireDate : ts];
    num = [NSDecimalNumber decimalNumberWithString : @"3210.50"];
    [e setSalary: num];

    // create a new Manager
    Manager* m2 = [[Manager alloc] init:db];
    [m2 setId:minId+3];
    [m2 setFirstName:@"Dagwood"];
    [m2 setLastName:@"Bumstead"];
    [m2 setComment:@"kill two birds with one stone"];
    [m2 setGender:@"Male"];
    ts = [NSDate dateWithString:@"2012-02-06 00:00:00 +0000"];
    [m2 setHireDate:ts];
    num = [NSDecimalNumber decimalNumberWithString:@"4567.50"];
    [m2 setSalary:num];
    [m2 setReportsTo:m2];

    [e setReportsTo:m2]; // the manager

    printf("\nEnumerating objects...\n");
    printf("%d Persons in the database %s\n", [Person instanceNumber : db],
	   [[db description] UTF8String]);

    MtObjectEnumerator* piter = [Person instanceEnumerator : db];
    while ((x = (Person*)[piter nextObject]) != nil) {
      printf("%d %s %s is a %s\n", 
	     [x id],
	     [[x firstName] UTF8String],
	     [[x lastName] UTF8String],
	     [[[x getMtClass] mtFullName] UTF8String]);
    }
    [piter close];


    [db commit];

    [db close];

  }
  @catch (MtException *e) {
    printf("ERROR: %s: (code=%d) %s\n", 
	   [[e name] UTF8String], 
	   [e errorCode], 
	   [[e message] UTF8String] );
    NSLog(@"Stack trace: %@\n", [e callStackSymbols]);
  }
  @finally {
#if !__has_feature(objc_arc)
    [db release];
#endif
  }

  printf("Done\n");
}

void deleteObjects(NSString *hostname, NSString *dbname, NSString *dbnamespace)
{
  MtDatabase* db;

  printf("=========== Delete Objects ===========\n");

  @try {

    // The third argument to MtDatabase init is given so that the connection 
    // object can find the mapping between the Objective-C class Person and 
    // the schema class Person defined in the namespace defined by the 
    // 'dbnamespace' parameter.
    //
    MtDynamicObjectFactory *factory = [[MtDynamicObjectFactory alloc] init:@"" :dbnamespace];
    db = [(MtDatabase *)[MtDatabase alloc] init:hostname :dbname andFactory:factory];

    [db open];

    [db startTransaction];

    [[Person getClass:db] removeAllInstances];

    printf("%d Persons (%s) in the database %s\n", [Person instanceNumber : db],
	   [[[Person getClass:db] mtFullName] UTF8String],
	   [[db description] UTF8String]);

    [db commit];

    [db close];

  }
  @catch (MtException *e) {
    printf("ERROR: %s: (code=%d) %s\n", 
	   [[e name] UTF8String], 
	   [e errorCode], 
	   [[e message] UTF8String] );
    NSLog(@"Stack trace: %@\n", [e callStackSymbols]);
  }
  @finally {
#if !__has_feature(objc_arc)
    [db release];
#endif
  }

  printf("Done\n");
}

int main(int argc, char **argv)
{
  NSString *hostname, *dbname;
  
  if (argc < 3) {
    printf("Usage: %s <host> <database>\n", argv[0]);
    return -1;
  }
#if !__has_feature(objc_arc)
  NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
#else
  @autoreleasepool {
#endif

  hostname = [NSString stringWithCString:argv[1] encoding:NSUTF8StringEncoding]; 
  dbname = [NSString stringWithCString:argv[2] encoding:NSUTF8StringEncoding];

  createObjects(hostname, dbname, @"examples.objc_examples.multidbs.db_one", 1000);

  createObjects(hostname, dbname, @"examples.objc_examples.multidbs.db_two", 2000);

  deleteObjects(hostname, dbname, @"examples.objc_examples.multidbs.db_one");

  deleteObjects(hostname, dbname, @"examples.objc_examples.multidbs.db_two");

#if !__has_feature(objc_arc)
  // free pool
  [pool release];
#else
  } // @autoreleasepool
#endif

  return 0;
}
